namespace ResolveClassDef

open System
open System.CodeDom
open System.Reflection
open System.Web.UI
open Microsoft.FSharp.MLLib
open Microsoft.FSharp.Quotations
open Microsoft.FSharp.Quotations.Raw;;
open Microsoft.FSharp.Quotations.RawTypes;;

(*---------------------------------------------------------------------------*)
(* QUOTATIONS FOR CLASSES                                                    *)

module QuotExt : sig

	/// Member of the class 
	/// For working with this type use cmfCtor, cmfMethod, cmfField, cmfProp..
	type classMember;;

	/// Type for working with class members
	/// Query function tests whether member is of specified type
	/// Make function creates member
	type 'a classMemberFamily with
		member Query : classMember -> 'a option
    member Make : 'a -> classMember
  end;;

	/// Constructor declaration 
	/// - expression should be lambda expression
	val cmfCtor : (expr) classMemberFamily;;
	
	/// Method delcaration
	/// - method name, expression (should be lambda) and return type
	///   (types of parameters are stored in lambda expression)
	val cmfMethod : (string*expr*Type option) classMemberFamily;;
	
	/// Field declaration
	/// - field name, type and init expression
	val cmfField : (string*Type*expr option) classMemberFamily;;
	
	/// Property declaration
	/// - property name, getter and setter (both should be lambda) and type
	val cmfProp : (string*expr*expr*Type) classMemberFamily;;
	
	/// Structure that contains information about class -
	/// consists of name, name of base class and members
	type classInfo = {
			name:string;
			baseName:string option;
			members:classMember list;
		};;

	/// Creates classInfo structure from class name,
	/// base class name and list of members	
	val makeClassInfo : string -> (string option) -> (classMember list) -> classInfo;;
		
end;;


(*---------------------------------------------------------------------------*)
(* RESOLVE CLASS QUOTATION                                                   *)
open QuotExt;;

module ClassQuotations : sig
  val getClassFromType : Type -> classInfo
end